import { ColorDynamic, isColorDynamicProp, parseResponsiveProp, parseSpaceProp } from '@superdispatch/ui';
import styled from 'styled-components';
import { createRuleNormalizer } from "../utils/RuleNormalizer.js";

//
// Colors
//

function normalizeColor(input) {
  return isColorDynamicProp(input) ? ColorDynamic[input] : undefined;
}

//
// Space
//

function parseSpace(space) {
  return "".concat(parseSpaceProp(space), "px");
}

//
// Margins
//

function parseMargin(input) {
  if (input === 'auto') return input;
  var prefix = '';
  if (typeof input == 'string' && input.startsWith('-')) {
    prefix = '-';
    input = input.slice(1);
  }
  return prefix + parseSpace(input);
}

//
// Borders
//

var normalizeBorderRadius = /*#__PURE__*/createRuleNormalizer({
  none: 0,
  small: 4,
  medium: 8
});
var normalizeBorderWidth = /*#__PURE__*/createRuleNormalizer({
  none: 0,
  small: 1,
  medium: 2,
  large: 4
});

//
// Rules
//

var normalizers = {
  display: undefined,
  color: normalizeColor,
  backgroundColor: normalizeColor,
  borderColor: normalizeColor,
  borderTopColor: normalizeColor,
  borderLeftColor: normalizeColor,
  borderRightColor: normalizeColor,
  borderBottomColor: normalizeColor,
  borderWidth: normalizeBorderWidth,
  borderTopWidth: normalizeBorderWidth,
  borderLeftWidth: normalizeBorderWidth,
  borderRightWidth: normalizeBorderWidth,
  borderBottomWidth: normalizeBorderWidth,
  margin: parseMargin,
  marginTop: parseMargin,
  marginLeft: parseMargin,
  marginRight: parseMargin,
  marginBottom: parseMargin,
  padding: parseSpace,
  paddingTop: parseSpace,
  paddingLeft: parseSpace,
  paddingRight: parseSpace,
  paddingBottom: parseSpace,
  borderRadius: normalizeBorderRadius,
  borderTopLeftRadius: normalizeBorderRadius,
  borderTopRightRadius: normalizeBorderRadius,
  borderBottomLeftRadius: normalizeBorderRadius,
  borderBottomRightRadius: normalizeBorderRadius,
  fontSize: undefined,
  width: undefined,
  maxWidth: undefined,
  minWidth: undefined,
  height: undefined,
  maxHeight: undefined,
  minHeight: undefined,
  overflow: undefined,
  overflowY: undefined,
  overflowX: undefined,
  top: undefined,
  left: undefined,
  right: undefined,
  bottom: undefined,
  position: undefined,
  alignItems: undefined,
  justifyContent: undefined
};
function injectRule(styles, key, breakpoint, value, normalizer) {
  if (normalizer != null) value = normalizer(value);
  if (value != null) {
    var rules = styles[breakpoint];
    if (typeof rules != 'object') {
      rules = {};
      styles[breakpoint] = rules;
    }
    rules[key] = String(value);
  }
}

//
// Box
//

export var Box = /*#__PURE__*/styled.div.withConfig({
  displayName: "Box",
  componentId: "SD__sc-1bc600y-0"
})(props => {
  var {
    breakpoints
  } = props.theme;
  var xs = breakpoints.up('xs');
  var sm = breakpoints.up('sm');
  var md = breakpoints.up('md');
  var styles = {
    margin: 0,
    padding: 0,
    borderWidth: 0,
    borderStyle: 'solid'
  };
  for (var k in props) {
    if (Object.prototype.hasOwnProperty.call(props, k) && k in normalizers) {
      var key = k;
      var prop = props[key];
      if (prop != null) {
        var [mobile, tablet, desktop] = parseResponsiveProp(prop);
        var normalizer = normalizers[key];
        injectRule(styles, key, xs, mobile, normalizer);
        injectRule(styles, key, sm, tablet, normalizer);
        injectRule(styles, key, md, desktop, normalizer);
      }
    }
  }
  return styles;
});